/* Using mouse device, get a new screen coordinate and button number.
 * Button numbers must be the following values which correspond to the
 * following software meanings: 1 - left button 2 - middle button 3 -
 * right button.
 *
 * The 'draw' argument specifies what sort of rubber-banding should be
 * used:  none, a line or a box.
 */

extern "C" {
#include <time.h>
#include "includes.h"
}

#include <qapplication.h>
#include <qcursor.h>
#include "Window.h"
#include "local_cpp.h"

static int Get_mouse (
	enum DriverWindow::DrawFunction draw,
    int cx, int cy,                     /* current x and y */
    int *nx, int *ny,                   /* new x and y */
    int *button
)
{
	QMouseEvent *event ;
	struct timespec ts = {0, 50000} ;

	window->getButtonPress(DriverWindow::None, 0, 0) ;
#ifndef QWS
	window->setCursor(QCursor::crossCursor) ;
#endif
	if ( draw != DriverWindow::None ) {
		window->setMouseTracking(TRUE) ;
	}

	/*
	 * Sit in a loop alternately processing events and sleeping for 50ms
	 * until a button press is received.
	 */
	while ( (event=window->getButtonPress(draw, cx, cy)) == NULL ) {
		qApp->processEvents(50) ;
		nanosleep(&ts, NULL) ;
	}

	if ( draw != DriverWindow::None ) {
		window->setMouseTracking(FALSE) ;
	}
#ifndef QWS
	window->setCursor(QCursor::arrowCursor) ;
#endif

	*nx = event->x() ;
	*ny = event->y() ;

	switch ( event->button() ) {
	case Qt::LeftButton :
		*button = 1 ;
		break ;
	case Qt::MidButton :
		*button = 2 ;
		break ;
	case Qt::RightButton :
		*button = 3 ;
		break ;
	default :
		*button = 0 ;
		break ;
	}

    return 0;
}

/* Using mouse device, get a new screen coordinate and button number.
 * Button numbers must be the following values which correspond to the
 * following software meanings: 1 - left button 2 - middle button 3 -
 * right button
 * 
 * This is called directly by the application programs.
 * 
 * A "rubberband" box is used.  One corner is fixed at the (cx, cy)
 * coordinate.  The opposite coordinate starts out at (*nx, *ny) and
 * then tracks the mouse.  Upon button depression, the current
 * coordinate is returned in (*nx, *ny) and the button pressed is
 * returned in button. */

int Get_location_with_box (
    int cx, int cy,                     /* current x and y */
    int *nx, int *ny,                   /* new x and y */
    int *button
)
{
	Get_mouse(DriverWindow::Rectangle, cx, cy, nx, ny, button) ;
    return 0;
}

/* Using mouse device, get a new screen coordinate and button number.
 * Button numbers must be the following values which correspond to the
 * following software meanings: 1 - left button 2 - middle button 3 -
 * right button
 * 
 * This is called directly by the application programs.
 * 
 * A "rubberband" line is used.  One end is fixed at the (cx, cy)
 * coordinate. The opposite end starts out at (*nx, *ny) and then
 * tracks the mouse. Upon button depression, the current coordinate is
 * returned in (*nx, *ny) and the button pressed in returned in
 * *button. */

int Get_location_with_line (
    int cx, int cy,                     /* current x and y */
    int *nx, int *ny,                   /* new x and y */
    int *button
)
{
	Get_mouse(DriverWindow::Line, cx, cy, nx, ny, button) ;
    return 0;
}

/* Using mouse device, get a new screen coordinate and button number.
 * Button numbers must be the following values which correspond to the
 * following software meanings: 1 - left button 2 - middle button 3 -
 * right button
 * 
 * This is called directly by the application programs.
 * 
 * A "crosshair" pointer is used. Upon button depression, the current
 * coordinate is returned in (*wx, *wy) and the button pressed in
 * returned in *button. */

int Get_location_with_pointer (int *wx, int *wy, int *button)
{
	Get_mouse(DriverWindow::None, 0, 0, wx, wy, button) ;
    return 0;
}
