extern "C" {
#include "gis.h"
#include <stdio.h>
#include <stdlib.h>
#include "includes.h"
#include "colors.h"
#include "driver.h"
#include "driverlib.h"
}

#include <qimage.h>
#include <qpixmap.h>
#include "Window.h"
#include "local_cpp.h"

/* Write 'nrows' lines of 'num' pixels contained as ints in 'array' to
 * the screen starting at the current position. 'withzeros' indicates
 * that zero-valued pixels in array should either be written or left
 * unchanged. 'color_type' indicates that the pixel values should
 * either be written directly to the screen or used to reference a
 * color look-up table. */

static QImage *image ;
static QPixmap *pixmap ;
static int alloc ;

int 
Raster_int(int num, int nrows, int *array, int withzeros, int color_type)
{
    int i;
	int (*assign_color)(int) ;

	if ( alloc < num ) {
		/* allocated image and pixmap aren't big enough, make new ones */
		if ( image ) {
			delete image ;
		}
		if ( pixmap ) {
			delete pixmap ;
		}
		image = new QImage(num, 1, 32) ;
		pixmap = new QPixmap(num, 1) ;
		alloc = num ;
	}

	if (color_type) {
		assign_color = Color ;
	}
	else {
		assign_color = color ;
	}

    /* If zeros are to be included, an entire raster row can be constructed */
    if ( withzeros ) {
	    for ( i=0; i<num; ++i ) {
			assign_color(array[i]) ;
			image->setPixel(i, 0, (int)current_color) ;
		}

		pixmap->convertFromImage(*image, 0) ;

		for ( i=0; i<nrows; ++i ) {
			bitBlt(window, cur_x, cur_y+i, pixmap) ;
			bitBlt(window->getBuffer(), cur_x, cur_y+i, pixmap) ;
		}
    }
    /* If zeros are not included may need to draw many shorter rasters.
     * If the pixel value in array is zero we don't disturb the
     * existing pixel of the drawable. If the pixel is non-zero we
     * re-write it. */
    else {
        int j, start_col, width;

        start_col = 0;
        width = 0;
        for ( j=0; j < num; j++ ) {
            if ( array[j] == 0 ) {
                if ( width > 0 ) {
					pixmap->convertFromImage(*image, 0) ;
                    for ( i=0; i < nrows; i++) {
						bitBlt(window, cur_x+start_col, cur_y+i, pixmap,
									0, 0, width, 1) ;
						bitBlt(window->getBuffer(), cur_x+start_col, cur_y+i,
									pixmap, 0, 0, width, 1) ;
                    }
                    width = 0;
                    start_col = j + 1;
                }
				else {
                    start_col++;
                }
            }
			else {            /* non-zero pixel, put into the image */
				assign_color(array[j]) ;
				image->setPixel(width++, 0, (int)current_color) ;
            }
        }

        /* Flush out any remaining data */
        if (width > 0) {
			pixmap->convertFromImage(*image, 0) ;
			for (i = 0; i < nrows; i++) {
				bitBlt(window, cur_x+start_col, cur_y+i, pixmap,
						0, 0, width, 1) ;
				bitBlt(window->getBuffer(), cur_x+start_col, cur_y+i,
						pixmap, 0, 0, width, 1) ;
			}
        }
    }
    return 1;
}
