extern "C" {
#include <sys/select.h>
#include <stdlib.h>
}

#include "Socket.h"

#define REC(a,b)    if ((eofile=rec((a),(b)))) break
#define RECTEXT(x,s)  if ((eofile=rectext(&x,&s))) break
#define SEND(a,b)   _send((a),(b))
#define SENDTEXT(x)  sendtext((x))

#ifdef QWS
#define PROCESS_BARRIER	if ( !doit ) break
#else
#define PROCESS_BARRIER
#endif

DriverSocket::DriverSocket()
{
	current_command = 0 ;
	inbuf = NULL ;
	atbuf = n_read = maxbuf = 0 ;
	start = 0 ;
	eofile = 0 ;
}

DriverSocket::~DriverSocket()
{
	if ( inbuf ) {
		free(inbuf) ;
	}
}

int DriverSocket::read1(char *c)
{
	if ( atbuf == n_read ) {
#ifndef QWS
		atbuf = 0 ;

		if ( bytesAvailable() == 0 ) {
			waitForMore(-1) ;
		}

		n_read = readBlock(inbuf, sizeof inbuf) ;

		if ( n_read < 0 ) {
			fprintf(stderr, "Monitor: read1: Error reading input\n") ;
			return 1 ;
		}
#else
		return 1 ;
#endif
	}

	*c = inbuf[atbuf++] ;
	return 0 ;
}

int DriverSocket::get1(char *c)
{
	if (read1(c) != 0) {
		return 1;		/* EOF */
	}
	if (*c != COMMAND_ESC) {
		return 0;		/* OK */
	}
	if (read1(c) != 0) {
		return 1;		/* EOF */
	}
	if ( *c ) {
		current_command = *c;
		return -1 ;		/* Got command within data */
	}

	*c = COMMAND_ESC ;	/* sequence COMMAND_ESC,0 becomes data COMMAND_ESC */
	return 0 ;			/* OK */
}

int DriverSocket::rec(void *buf, int n)
{
	char *cbuf = (char *)buf;
	int stat;

	while ( n-- > 0 ) {
		if ((stat=get1(cbuf++)) != 0) {
			return stat ; /* EOF or COMMAND_ESC */
		}
	}
	return 0 ;
}

int DriverSocket::rectext(char **buff_p, int *size_p)
{
	char *buff = *buff_p;
	int size = *size_p; 
	int i, stat;

	for ( i=0 ; ; i++ ) {
		char c ;

		stat = get1(&c);
		if (stat != 0) {
			return stat; /* EOF or COMMAND_ESC */
		}

		if (i >= size) {
			*size_p = size = size ? size * 2 : 1000;
			*buff_p = buff = (char *)G_realloc(buff, size);
		}

		buff[i] = c;

		if ( !c ) {
			return 0 ;
		}
	}
	return 0;
}

int DriverSocket::_send(void *buf, int n)
{
	int r = writeBlock((char *)buf, n) ;
	if ( r < 0 ) {
		fprintf(stderr, "Monitor: _send writeBlock failed\n") ;
		return 1 ;
	}

	if ( r < n ) {
		fprintf(stderr, "Monitor: _send writeBlock short count %d of %d\n",
					r, n) ;
		return 1 ;
	}

	return 0 ;
}

int DriverSocket::sendtext(char *s)
{
	SEND(s, strlen(s) + 1);
	return 0 ;
}

int DriverSocket::RESULT(int n)
{
	unsigned char c;

	c = n;
	SEND((char *)&c, 1);

	return 0;
}

int DriverSocket::get_command(char *c)
{
	/* is there a command char pending? */
	if ((*c = current_command)) {
		current_command = 0;
		return 0;
	}

	/*
	 * look for 1 (or more) COMMAND_ESC chars
	 * followed by a non-zero comamnd token char
	 */
	while (read1(c) == 0) {  /* while !EOF */
		if (*c != COMMAND_ESC) {
			continue;
		}
		while (*c == COMMAND_ESC) {
			if (read1(c) != 0) {
				fprintf(stderr, "Monitor: get_command: Premature EOF\n");
				return 1;               /* EOF */
			}
		}
		if (*c) {
			return 0;           /* got the command token */
		}
	}

	return 1 ;
}

static PAD *curpad;

static unsigned char *blua;
static unsigned char *grna;
static unsigned char *reda;
static unsigned char *nula;
static int blu_alloc;
static int grn_alloc;
static int red_alloc;
static int nul_alloc;

static int *xarray;
static int *yarray;
static int n_xarray;
static int n_yarray;

#ifdef QWS
int DriverSocket::process_command(char c, bool doit)
#else
int DriverSocket::process_command(char c)
#endif
{
	static char *name;
	static int name_size;
	static char *text;
	static int text_size;

	int t, b, l, r;
	int x, y;
	unsigned char red, grn, blu;
	int min, max;
	int number;
	int index;
	int button;
	float wx;
	ITEM *item;
	LIST *list;
	PAD *pad ;
	unsigned char ch;

	switch(c) {
	case BEGIN:
		PROCESS_BARRIER ;
		ch = 0;
		for (index = -10; index < BEGIN_SYNC_COUNT; index++)
			SEND(&ch, 1);
		ch = COMMAND_ESC;
		SEND(&ch, 1);
		break;
	case RESPOND:
		PROCESS_BARRIER ;
		Respond();
		SEND(&ch, 1);
		break;
	case GET_NUM_COLORS:
		PROCESS_BARRIER ;
		Number_of_colors(&index);
		SEND(&index, sizeof index);
		break;
	case STANDARD_COLOR:
		REC(&index, sizeof index);
		PROCESS_BARRIER ;
		Standard_color(index);
		break;
	case COLOR:
		REC(&index, sizeof index);
		PROCESS_BARRIER ;
		Color(index);
		break;
	case RGB_COLOR:
		REC(&red, sizeof red);
		REC(&grn, sizeof grn);
		REC(&blu, sizeof blu);
		PROCESS_BARRIER ;
		RGB_color(red, grn, blu);
		break;
	case COLOR_TABLE_FIXED:
		PROCESS_BARRIER ;
		x = Color_table_fixed();
		SEND(&x, sizeof x);
		break;
	case COLOR_TABLE_FLOAT:
		PROCESS_BARRIER ;
		x = Color_table_float();
		SEND(&x, sizeof x);
		break;
	case COLOR_OFFSET:
		REC(&index, sizeof index);
		PROCESS_BARRIER ;
		Color_offset(index);
		break;
	case COLOR_PRINT:
		break;
	case CONT_ABS:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Cont_abs(x, y);
		break;
	case CONT_REL:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Cont_rel(x, y);
		break;
	case BOX_ABS:
		REC(&l, sizeof l);
		REC(&t, sizeof t);
		REC(&r, sizeof r);
		REC(&b, sizeof b);
		PROCESS_BARRIER ;
		Box_abs(l, t, r, b);
		break;
	case BOX_REL:
		REC(&l, sizeof l);
		REC(&t, sizeof t);
		PROCESS_BARRIER ;
		Box_rel(l, t);
		break;
	case ERASE:
		PROCESS_BARRIER ;
		Erase();
		break;
	case GET_LOCATION_WITH_BOX:
		REC(&t, sizeof t);
		REC(&b, sizeof b);
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Get_location_with_box(t, b, &x, &y, &button);
		SEND(&x, sizeof x);
		SEND(&y, sizeof y);
		SEND(&button, sizeof button);
		break;
	case GET_LOCATION_WITH_LINE:
		REC(&t, sizeof t);
		REC(&b, sizeof b);
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Get_location_with_line(t, b, &x, &y, &button);
		SEND(&x, sizeof x);
		SEND(&y, sizeof y);
		SEND(&button, sizeof button);
		break;
	case GET_LOCATION_WITH_POINTER:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		REC(&button, sizeof button);
		PROCESS_BARRIER ;
		Get_location_with_pointer(&x, &y, &button);
		SEND(&x, sizeof x);
		SEND(&y, sizeof y);
		SEND(&button, sizeof button);
		break;
	case GRAPH_CLOSE:
		PROCESS_BARRIER ;
		Graph_Close();
		exit(0);
	case LINEMOD:
		REC(&index, sizeof index);
		PROCESS_BARRIER ;
		Linemod(&index);
		break;
	case MOVE_ABS:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Move_abs(x, y);
		break;
	case MOVE_REL:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Move_rel(x, y);
		break;
	case RASTER_CHAR:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		REC(&index, sizeof index);
		blua = (unsigned char *) xalloc(blua, &blu_alloc, x, sizeof(*blua));
		REC(blua, x * sizeof(char));
		PROCESS_BARRIER ;
		if (index != 0) index = 1;
		Raster_char(x, y, blua, index, 1);
		break;
	case RASTER_INT:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		REC(&index, sizeof index);
		xarray = (int *) xalloc(xarray, &n_xarray, x, sizeof(*xarray));
		REC(xarray, x * sizeof(*xarray));
		PROCESS_BARRIER ;
		if (index != 0) index = 1;
		Raster_int(x, y, xarray, index, 1);
		break;
	case RGB_RASTER:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		reda = (unsigned char *) xalloc(reda, &red_alloc, x, sizeof(*reda));
		grna = (unsigned char *) xalloc(grna, &grn_alloc, x, sizeof(*grna));
		blua = (unsigned char *) xalloc(blua, &blu_alloc, x, sizeof(*blua));
		nula = (unsigned char *) xalloc(nula, &nul_alloc, x, sizeof(*nula));
		REC(reda, x * sizeof(char));
		REC(grna, x * sizeof(char));
		REC(blua, x * sizeof(char));
		REC(nula, x * sizeof(char));
		REC(&t, sizeof t);
		PROCESS_BARRIER ;
		RGB_raster(x, y, reda, grna, blua, t ? nula : NULL);
		break;
	case RGB_COLORS:
		reda = (unsigned char *) xalloc(reda, &red_alloc, 256, sizeof(*reda));
		grna = (unsigned char *) xalloc(grna, &grn_alloc, 256, sizeof(*grna));
		blua = (unsigned char *) xalloc(blua, &blu_alloc, 256, sizeof(*blua));
		REC(reda, 256);
		REC(grna, 256);
		REC(blua, 256);
		PROCESS_BARRIER ;
		Set_RGB_color(reda, grna, blua);
		break;
	case POLYGON_ABS:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polygon_abs(xarray, yarray, number);
		break;
	case POLYGON_REL:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polygon_rel(xarray, yarray, number);
		break;
	case POLYLINE_ABS:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polyline_abs(xarray, yarray, number);
		break;
	case POLYLINE_REL:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polyline_rel(xarray, yarray, number);
		break;
	case POLYDOTS_ABS:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polydots_abs(xarray, yarray, number);
		break;
	case POLYDOTS_REL:
		REC(&number, sizeof number);
		xarray = (int *) xalloc(xarray, &n_xarray, number, sizeof(*xarray));
		yarray = (int *) xalloc(yarray, &n_yarray, number, sizeof(*yarray));
		REC(xarray, number * sizeof(xarray[0]));
		REC(yarray, number * sizeof(yarray[0]));
		PROCESS_BARRIER ;
		Polydots_rel(xarray, yarray, number);
		break;
	case RESET_COLORS:
		REC(&min, sizeof min);
		REC(&max, sizeof max);
		number = max - min + 1;
		reda = (unsigned char *) xalloc(reda, &red_alloc, number, sizeof(*reda));
		grna = (unsigned char *) xalloc(grna, &grn_alloc, number, sizeof(*grna));
		blua = (unsigned char *) xalloc(blua, &blu_alloc, number, sizeof(*blua));
		REC(reda, number * sizeof(char));
		REC(grna, number * sizeof(char));
		REC(blua, number * sizeof(char));
		PROCESS_BARRIER ;
		Reset_colors(min, max, reda, grna, blua);
		break;
	case RESET_COLOR:
		REC(&red, sizeof red);
		REC(&grn, sizeof grn);
		REC(&blu, sizeof blu);
		REC(&number, sizeof number);
		PROCESS_BARRIER ;
		Reset_color(red, grn, blu, number);
		break;
	case SCREEN_LEFT:
		PROCESS_BARRIER ;
		Screen_left(&index);
		SEND(&index, sizeof index);
		break;
	case SCREEN_RITE:
		PROCESS_BARRIER ;
		Screen_rite(&index);
		SEND(&index, sizeof index);
		break;
	case SCREEN_BOT:
		PROCESS_BARRIER ;
		Screen_bot(&index);
		SEND(&index, sizeof index);
		break;
	case SCREEN_TOP:
		PROCESS_BARRIER ;
		Screen_top(&index);
		SEND(&index, sizeof index);
		break;
	case SET_WINDOW:
		REC(&t, sizeof t);
		REC(&b, sizeof b);
		REC(&l, sizeof l);
		REC(&r, sizeof r);
		PROCESS_BARRIER ;
		Set_window(t, b, l, r);
		break;
	case GET_TEXT_BOX:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		Get_text_box(text, &t, &b, &l, &r);
		SEND(&t, sizeof t);
		SEND(&b, sizeof b);
		SEND(&l, sizeof l);
		SEND(&r, sizeof r);
		break;
	case FONT:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		x = Font_get(text);
		SEND(&x, sizeof x);
		break;
	case TEXT:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		Text(text);
		break;
	case TEXT_SIZE:
		REC(&x, sizeof x);
		REC(&y, sizeof y);
		PROCESS_BARRIER ;
		Text_size(x, y);
		break;
	case TEXT_ROTATION:
		REC(&wx, sizeof wx);
		PROCESS_BARRIER ;
		Text_rotation(wx);
		break;
	case PANEL_SAVE:
		RECTEXT(text, text_size);
		REC(&t, sizeof t);
		REC(&b, sizeof b);
		REC(&l, sizeof l);
		REC(&r, sizeof r);
		PROCESS_BARRIER ;
		Panel_save(text, t, b, l, r);
		break;
	case PANEL_RESTORE:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		Panel_restore(text);
		break;
	case PANEL_DELETE:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		Panel_delete(text);
		break;
	case PAD_CREATE:
		RECTEXT(text, text_size);
		PROCESS_BARRIER ;
		if (*text == 0) /* this is scratch pad */
			RESULT(OK);
		else if (find_pad(text) != NULL)
			RESULT(DUPLICATE); /* duplicate pad */
		else if (create_pad(text))
			RESULT(OK);
		else
			RESULT(NO_MEMORY);
		break;

	case PAD_CURRENT:
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			SENDTEXT("");
		}
		else {
			RESULT(OK);
			SENDTEXT(curpad->name);
		}
		break;
	case PAD_DELETE:
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
		}
		else if (*curpad->name == 0) {
			RESULT(ILLEGAL);
		}
		else {
			delete_pad(curpad);
			curpad = NULL;
			RESULT(OK);
		}
		break;
	case PAD_INVENT:
		PROCESS_BARRIER ;
		invent_pad(text);
		SENDTEXT(text);
		break;
	case PAD_LIST:
		PROCESS_BARRIER ;
		for (pad = pad_list(); pad != NULL; pad = pad->next)
			if (*pad->name)
				SENDTEXT(pad->name);
			SENDTEXT("");
		break;

	case PAD_SELECT:
		RECTEXT(text, text_size); /* pad name */
		PROCESS_BARRIER ;
		curpad = find_pad(text);
		if (curpad == NULL)
			RESULT(NO_PAD);
		else
			RESULT(OK);
		break;

	case PAD_GET_ITEM:
		RECTEXT(text, text_size); /* item name */
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			break;
		}
		item = find_item(curpad, text);
		if (item == NULL) {
			RESULT(NO_ITEM);
			break;
		}
		RESULT(OK);
		for (list = item->list; list != NULL; list = list->next) {
			if (*list->value) {
				SENDTEXT(list->value);
			}
		}
		SENDTEXT("");
		break;

	case PAD_SET_ITEM:
		RECTEXT(name, name_size); /* item name */
		RECTEXT(text, text_size); /* item value */
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			break;
		}
		delete_item(curpad, name);
		if (append_item(curpad, name, text, 0))
			RESULT(OK);
		else
			RESULT(NO_MEMORY);
		break;

	case PAD_APPEND_ITEM:
		RECTEXT(name, name_size); /* item name */
		RECTEXT(text, text_size); /* item value */
		REC(&index, sizeof index); /* replace flag */
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			break;
		}
		if (append_item(curpad, name, text, index)) {
			RESULT(OK);
		}
		else {
			RESULT(NO_MEMORY);
		}
		break;

	case PAD_DELETE_ITEM:
		RECTEXT(text, text_size); /* item name */
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			break;
		}
		delete_item(curpad, text);
		RESULT(OK);
		break;

	case PAD_LIST_ITEMS:
		PROCESS_BARRIER ;
		if (curpad == NULL) {
			RESULT(NO_CUR_PAD);
			break;
		}
		RESULT(OK);
		for (item = curpad->items; item != NULL; item = item->next) {
			if (*item->name) {
				SENDTEXT(item->name);
			}
		}
		SENDTEXT("");
		break;

	default:
		fprintf(stderr, "\nUnknown command: %d last: %d\n", c, lc);
		break;
	}
	lc = c;

	return eofile;
}

#ifdef QWS
int DriverSocket::process_data()
{
	char c ;
	int i, len, last, end ;

	len = bytesAvailable() ;
	inbuf = xalloc(inbuf, &maxbuf, len+start, 1);
	n_read = readBlock(inbuf+start, len) ;

	if ( n_read < 0 ) {
		fprintf(stderr, "Monitor: read1: Error reading input\n") ;
		return 1 ;
	}

	// find the index of the last known command in the buffer
	last = 0 ;
	n_read += start ;
	for ( i=0; i<n_read-1; ++i ) {
		if ( inbuf[i] == COMMAND_ESC && inbuf[i+1] != 0 ) {
			last = i ;
		}
	}

	// attempt to read last command
	atbuf = last ;
	if ( get_command(&c) != 0 ) {
		fprintf(stderr, "get_command failed\n") ;
		return 1 ;
	}

	// end is the end of the data in the buffer, n_read will be the end of
	// the last complete command
	end = n_read ;
	if ( process_command(c, FALSE) ) {
		// command is incomplete, don't attempt to process it
		n_read = last ;
	}
	else if ( inbuf[n_read-1] == COMMAND_ESC ) {
		// last command was complete, but there's part of the start of a
		// new command in the last byte of the buffer
		--n_read ;
	}

	// process all complete commands in the buffer
	atbuf = 0 ;
	while ( atbuf != n_read ) {
		if ( get_command(&c) != 0 ) {
			fprintf(stderr, "get_command failed\n") ;
			return 1 ;
		}

		if ( process_command(c, TRUE) ) {
			fprintf(stderr, "Monitor: premature EOF\n") ;
			return 1 ;
		}
	}

	// move partial command from the end of the buffer to the start
	for ( i=n_read; i<end; ++i ) {
		inbuf[i-n_read] = inbuf[i] ;
	}
	start = end-n_read ;

	return 0 ;
}
#else
int DriverSocket::process_data()
{
	char c ;

	inbuf = xalloc(inbuf, &maxbuf, bytesAvailable(), 1);
	n_read = readBlock(inbuf, maxbuf) ;

	if ( n_read < 0 ) {
		fprintf(stderr, "Monitor: read1: Error reading input\n") ;
		return 1 ;
	}

	atbuf = 0 ;
	while ( atbuf != n_read || bytesAvailable() ) {
		if ( get_command(&c) != 0 ) {
			fprintf(stderr, "get_command failed\n") ;
			return 1 ;
		}

		if ( process_command(c) ) {
			fprintf(stderr, "Monitor: premature EOF\n") ;
			return 1 ;
		}
	}

	return 0 ;
}
#endif
