extern "C" {
#include <stdio.h>
#include "driver.h"
#include "driverlib.h"
#include "colors.h"
#include "pad.h"
}

#include "Window.h"

DriverWindow::DriverWindow()
{
	setBackgroundMode(NoBackground) ;
	initialised = 0 ;
	buttonPress = NULL ;
	windowPainter = NULL ;
	bufferPainter = NULL ;

	lastx = lasty = 0 ;
	origx = origy = 0 ;
	drawn = FALSE ;
	drawFunction = DriverWindow::None ;
}

void DriverWindow::paintEvent(QPaintEvent *event)
{
	// copy from backing store to the window
	if ( bufferPainter && bufferPainter->isActive() ) {
		bufferPainter->end() ;
	}
	bitBlt(this, event->rect().topLeft(), &buffer, event->rect()) ;
}

int _time_stamp (PAD *pad)
{
    append_item(pad,"time","1",1);

    return 0;
}

void DriverWindow::resizeEvent(QResizeEvent *event)
{
	PAD *curpad, *nextpad;
	char buf[64];

	if ( initialised && screen_right == event->size().width() &&
			screen_bottom == event->size().height() ) {
		return ;
	}
	else {
		initialised = 1 ;
	}

	/* set the window size */
	screen_right = event->size().width() ;
	screen_bottom = event->size().height() ;
	Set_window(screen_top, screen_bottom, screen_left, screen_right) ;

	/* resize backing store */
	buffer.resize(event->size()) ;

	/* set standard color to black and erase */
	Standard_color(BLACK);
	Erase();

	/* delete the time and current window out of the scratch pad */
	curpad = find_pad("");
	delete_item(curpad,"time");
	delete_item(curpad,"cur_w");

	/* delete all other pads */
	for ( curpad=pad_list(); curpad != NULL; curpad=nextpad ) {
		nextpad = curpad->next;
		if ( *curpad->name ) {
			delete_pad(curpad);
		}
	}

	curpad = NULL;

    /* Dnew("full_screen") */
    /* find a pad called "full_screen" */
    create_pad("full_screen");
    sprintf(buf,"%d %d %d %d",
	    screen_top,screen_bottom,screen_left,screen_right);
    curpad = find_pad("full_screen");
    append_item(curpad, "d_win", buf, 0);
    _time_stamp(curpad);
    /* Dchoose("full_screen") */

    /* set the time and window name in no-name pad */
    curpad = find_pad("");
    append_item(curpad, "cur_w", "full_screen", 0);
    _time_stamp(curpad);
}

QPainter *DriverWindow::getBufferPainter()
{
	if ( bufferPainter ) {
		delete bufferPainter ;
	}

	bufferPainter = new QPainter(&buffer) ;
	return bufferPainter ;
}

QPainter *DriverWindow::getWindowPainter()
{
	if ( windowPainter ) {
		delete windowPainter ;
	}

	windowPainter = new QPainter(this) ;
	return windowPainter ;
}

QPixmap *DriverWindow::getBuffer()
{
	return &buffer ;
}

void DriverWindow::mousePressEvent(QMouseEvent *event)
{
	if ( buttonPress ) {
		delete buttonPress ;
	}

	buttonPress = new QMouseEvent(QEvent::MouseButtonPress, event->pos(),
									event->globalPos(), event->button(),
									event->state()) ;
}

void DriverWindow::drawLine(int x1, int y1, int x2, int y2, QPainter *painter)
{
	painter->drawLine(x1, y1, x2, y2) ;
}

void DriverWindow::drawRect(int x1, int y1, int x2, int y2, QPainter *painter)
{
	int tmp ;

	if ( x1 > x2 ) {
		tmp = x1 ;
		x1 = x2 ;
		x2 = tmp ;
	}

	if ( y1 > y2 ) {
		tmp = y1 ;
		y1 = y2 ;
		y2 = tmp ;
	}

	painter->drawRect(x1, y1, x2-x1, y2-y1) ;
}

void DriverWindow::drawRubberBand(QMouseEvent *event, bool draw)
{
	void (*drawer)(int,int,int,int,QPainter*) ;

	switch (drawFunction) {
	case Rectangle :
		drawer = drawRect ;
		break ;
	case Line :
		drawer = drawLine ;
		break ;
	default :
		return ;
	}

	getWindowPainter() ;
	windowPainter->setRasterOp(Qt::XorROP) ;
	windowPainter->setPen(darkBlue) ;
	if ( drawn ) {
		drawer(origx, origy, lastx, lasty, windowPainter) ;
	}
	if ( draw ) {
		drawer(origx, origy, event->x(), event->y(), windowPainter) ;
	}

	if ( event ) {
		lastx = event->x() ;
		lasty = event->y() ;
	}
	drawn = draw ;
}

void DriverWindow::mouseMoveEvent(QMouseEvent *event)
{
	drawRubberBand(event, TRUE) ;
}

QMouseEvent *DriverWindow::getButtonPress(enum DrawFunction draw, int x, int y)
{
	origx = x ;
	origy = y ;
	drawFunction = draw ;

	QMouseEvent *event = buttonPress ;

	if ( buttonPress != NULL ) {
		drawRubberBand(NULL, FALSE) ;
	}

	buttonPress = NULL ;
	return event ;
}
