extern "C" {
#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "gis.h"
#include "graph.h"
#include "driverlib.h"
#include "driver.h"
#include "pad.h"
#include "winname.h"
}

#ifdef QWS
#include <qpe/qpeapplication.h>
#else
#include <qapplication.h>
#endif
#include <qmessagebox.h>
#include <qcolor.h>
#include "Server.h"
#include "Window.h"
#include "local_cpp.h"

int NCOLORS;

int screen_left;
int screen_right;
int screen_bottom;
int screen_top;

int cur_x;
int cur_y;

double text_size_x;
double text_size_y;
double text_rotation;

DriverWindow *window ;

int main(int argc, char **argv)
{
    const char *p;
    char *me;
    char *connpath;

	char *title, *format ;
    pid_t pid;
    int foreground;
#ifdef USE_G_SOCKS
    int listenfd;
#endif

    /* The calling syntax is as follows:
          monitor_name [-] "input_fifo output_fifo"

       The "-", if present, causes the monitor to run in foreground.
       Otherwise, once it has been determined that the monitor is not
       already running, we will fork and the parent will exit, so that
       the monitor is left running in background.  For more information,
       see the comments in Dstart_mon.c.
    */

    if (argc != 4)
    {
        fprintf(stderr,"Usage:  %s <name> [-] \"input_fifo output_fifo\"\n", argv[0]);
		QMessageBox::warning(0, "QTDRIVER",  "Failed to run driver") ;
        return 1;
    }

    /* whoami */
    me = argv[1];

    foreground = (argv[2][0] == '-');

#ifdef USE_G_SOCKS
    connpath = G_sock_get_fname(me);
    if (connpath == NULL)
	    G_fatal_error("In %s: Couldn't get socket path.\n", __FILE__);
#else
    connpath = argv[3];
#endif

    /* Syntax is all right.  Now we must check and see whether or not someone */
    /* (possibly another invocation of ourself) is using our socket/FIFOs.    */

    if (check_connection(me, connpath) != 0)
    {
	/* Our socket/FIFOs are in use (or other error) */
	G_fatal_error("Unable to start monitor <%s>", me);
    }

    /* initialize graphics */

    p = getenv ("GRASS_WIDTH");
    screen_left = 0;
    screen_right = (p && atoi(p)) ? atoi(p) : DEF_WIDTH;

    p = getenv ("GRASS_HEIGHT");
    screen_top = 0;
    screen_bottom = (p && atoi(p)) ? atoi(p) : DEF_HEIGHT;

    if (Graph_Set(argc, argv) < 0)
        exit(1);

    /* Initialize color stuff */
    Color_table_fixed();

    /* We are free to run now.  No one is using our socket/FIFOs.             */
    /* If we are to run in background, we will have foreground == 0 from      */
    /* the syntax check above. */

#ifdef USE_G_SOCKS
    listenfd = prepare_connection_sock(me, connpath);
#endif

    /* initialize the pads */
    create_pad("");    /* scratch pad */

    fprintf(stderr,"Graphics driver [%s] started\n", me);

    if (!foreground)
    {
        if ( (pid=fork()) != 0 )
        {
            if (pid > 0)        /* parent exits */
            {
                exit(0);
            }
            else    /* weren't able to fork */
            {
                fprintf(stderr,"Error - Could not fork to start [%s]\n",me);
                exit(1);
            }
        }
	else
	{
	/* change process groups to be shielded from keyboard signals */
	/* note: use BSD form of call, which will also work for ATT   */
#ifdef SETPGRP_VOID
	    setpgrp();
#else
	    setpgrp(0, getpid());
#endif
	}
    }               /* monitor runs */

	QApplication::setColorSpec(QApplication::ManyColor) ;
#ifdef QWS
	QPEApplication myapp(argc, argv) ;
#else
	QApplication myapp(argc, argv) ;
#endif

	DriverServer server(listenfd) ;

	window = new DriverWindow() ;
	window->resize(screen_right, screen_bottom) ;

	format = "GRASS %s - Monitor: %s" ;
	title = (char *)malloc(strlen(WIN_NAME)+strlen(me)+strlen(format)) ;
	sprintf(title, format, WIN_NAME, me);
	window->setCaption(title) ;

#ifdef QWS
	myapp.showMainWidget(window) ;
#else
	myapp.setMainWidget(window) ;
	window->show() ;
#endif

    return myapp.exec();
}
